/*
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
************************************************** 603_Projekt_IRDMS ***************************************************
------------------------------------------------------------------------------------------------------------------------
**************************************************** main.c ** V1.6 ****************************************************
------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------
 Pegelmessung mittels IRDMS
 Zusaetzliche Header..: keine
 Module (*.c und *.h).: application, lcd_2wire, num_conversion
 Benoetigte Hardware..: RWNA-Steuerung mit 20x4-LCD und 2-Draht-LCD-Interface, Sensorkopf mit IRDMS
 Test-Hardware........: Testboard, LED1 und LED2 zur Simulation der Schaltvorgaenge, 20x4-LCD und 2-Draht-LCD-Interface,
                        Sensorkopf mit IRDMS oder Testschaltung gem. Bild 7.3.3-28: IRDMS-Simulation (Testschaltung)
 Version..............: 1.6 (neue Daten fuer Leerlaufschutz der Motorpumpe)
 Compiler.............: CodeVisionAVR
 Chip.................: ATmega88
 Datum................: September 2012
 Autor................: Alwin Lenck (ALE23)
------------------------------------------------------------------------------------------------------------------------

Sharp IRDMS (InfraRed Distance Measurment Sensor) http://www.sharpsma.com
Die Sensoren arbeiten nach folgendem Prinzip: Der IR-Emitter sendet einen Infrarotstrahl aus. Trifft der Strahl auf
ein Objekt, so bildet er auf dem Objekt einen Bildpunkt, der durch die Empfangs-Optik auf dem linearen einzeiligen
PSD-Chip (Position Sensitive Detector) des IR-Detektors erkannt wird. Je nach Entfernung des Objektes ist das
Eintreffen des Strahls auf dem Chip oertlich verschieden und erzeugt ein entsprechendes eindeutiges Spannungssignal.

*** In diesem Projekt wird der Sensor GP2Y0A02YK0F mit einem Messbereich von 20cm bis 150cm eingesetzt
*** Eine detaillierte Beschreibung befindet sich im schriftlichen Teil 7.3.3 AVR-Projekt IRDMS

Anmerkung: Der ADC wird in diesem Projekt NICHT ueber ein Modul gesteuert, sondern in der main.c direkt initialisiert
------------------------------------------------------------------------------------------------------------------------
EINGESETZTE RESOURCEN

*** LCD 2-Draht-Interface:
CLOCK                       PB1 (aenderbar in "application.h")  Shift-Register-Takt
DATA                        PB2 (aenderbar in "application.h")  Daten bzw. Kommandos fuer die LCD-Uebergabe

*** LEDs:
Keine

*** Tasten:
Keine

*** Relais:
MAGNET                      PD0 Relais fuer Magnetventil; Ueberlauf in den Gartenteich (in Funktion np_pegel aenderbar)
MOTOR                       PD1 Relais fuer Pumpenaggregat; AUS bei Pegel-Minimum (in Funktion np_pegel aenderbar)

*** Analog Digital Converter (ADC):
Analog-Eingangs-Pin         PC5(AD5) Fuer zu konvertierende Spannung (0 ... 1,1 Volt; in Funktion adc_read aenderbar)

------------------------------------------------------------------------------------------------------------------------
TYPENFESTLEGUNG DURCH SYNONYM-BILDUNG

Es werden folgende Synonyme aus der Header-Datei typedefs.h benutzt:
typedef unsigned char U08;   // Bildung eines Synonyms: aus unsigned char wird U08
typedef unsigned char U16;   // Bildung eines Synonyms: aus unsigned int wird U16
typedef U08 __flash *fpU08;  // Bildung eines Synonyms: fpU08 steht fuer U08 __flash *...
                             // oder als Synonym fuer unsigned char __flash *... -
                             // das heisst fpU08 ist ein Pointer auf eine char-Variable im Flash-Speicher
*/
//----------------------------------------------------------------------------------------------------------------------
// Header-Dateien einfuegen:
//----------------------------------------------------------------------------------------------------------------------
#include "application.h"     // Initialisierung der Anwendung
#include "lcd_2wire.h"       // Initialisierung des 2Draht-LCDs; beinhaltet: "string_conversion.h"

//----------------------------------------------------------------------------------------------------------------------
//Funktions-Prototypen:
//----------------------------------------------------------------------------------------------------------------------
void rwna_init(void);        // Initialisierung der Regenwassernutzungsanlage
void adc_init(void);         // Initialisierung des ADC
void adc_read(void);         // Lesen eines neuen Messwertes via ADC
void nl_messen(void);        // Messwert-Erfassung (Mittelwertbildung aus 256 Messwerten)
void w_p_v_tabellen(void);   // Pegel-Wert und Volumen-Wert aus Werte-Tabelle durch Interpolation entnehmen
void ruhe(void);             // Wichtig fuer die Verbrauchsanzeigen: Warten auf "Beruhigung" bei den Pegel-Werten
void volumina(void);         // Ermitteln der Verbrauchs- und Ueberlaufs-Volumina
void lcd_display(void);      // Anzeige der Pegel-, Volumen-, Verbrauchs- und Ueberlaufs-Werte
void np_pegel(void);         // Pruefen auf Ueberlauf bei Pegel-Maximum und Motorschutz bei Pegel-Minimum

//----------------------------------------------------------------------------------------------------------------------
// Globale Variablen:
//----------------------------------------------------------------------------------------------------------------------
U08 i, j;                    // i, j      benutzte Zaehler
U08 vol   = 0;               // VOL       Logischer Schalter fuer Volumen-Update
U08 p_120 = 0;               // P120      Logischer Schalter fuer Pegel-Wert 120cm
S16 adc_einzel;              // AE        ADC-Einzelmesswert vom IRDMS
S16 adc_mittel;              // AM        ADC-Mittelwert gemittelt aus 32 ADC-Einzelmesswerten
S16 n_lesen;                 // NL        Neuer ADC-Lesewert gemittelt aus 8 aktuellen ADC-Mittelwerten
S16 w1_tab;                  // W1        Unterer Interpolationswert aus der ADC-Werte-Tabelle
S16 w2_tab;                  // W2        Oberer Interpolationswert aus der ADC-Werte-Tabelle

__flash S16 w[25];           // W[i]      ADC-Tabellenwert aus der ADC-Werte-Tabelle mit 25 abgestuften ADC-Werten
S16 p1_tab;                  // P1        Unterer Interpolationswert aus der Pegel-Tabelle
S16 p2_tab;                  // P2        Oberer Interpolationswert aus der Pegel-Tabelle
__flash S16 p[25];           // P[i]      Pegel-Tabellenwert aus der Pegel-Tabelle mit 25 abgestuften Pegelwerten
float v1_tab;                // V1        Unterer Interpolationswert aus der Volumen-Tabelle
float v2_tab;                // V2        Oberer Interpolationswert aus der Volumen-Tabelle
__flash float v[25];         // V[i]      Volumen-Tabellenwert aus der Volumen-Tabelle mit 25 abgestuften Volumenwerten
float prop_fak;              // pro       Proportionalitaetsfaktor zwischen den Tabellenwerten
S16 n_pegel;                 // NP        Neuer Mittelwert in Pegel-cm (aus Pegel-Tabelle)
S16 a_pegel;                 // AP        Alter Mittelwert in Pegel-cm aus vorherigem Volumen-Update

float n_volumen ;            // NV        Neuer Mittelwert in Volumen-Kubikmeter (aus Volumen-Tabelle)
float a_volumen ;            // AV        Alter Mittelwert in Volumen-Kubikmeter aus vorherigem Volumen-Update
float d_volumen ;            // DV        Differenz zwischen dem alten und dem neuen Volumenwert
float t_volumen ;            // TV        Summe Teich-Ueberlauf in Volumen-Kubikmeter
float g_volumen ;            // GV        Summe Gartenwasser-Verbrauch in Volumen-Kubikmeter

//----------------------------------------------------------------------------------------------------------------------
// ADC-Werte-Tabelle der 25 ADC-Werte fuer die Abstaende 140cm bis 20cm in 5cm-Intervallen:
//----------------------------------------------------------------------------------------------------------------------
__flash S16 w[25] =
  {   177,   180,   190,   196,   213,
      216,   228,   236,   244,   258,
      273,   294,   310,   322,   346,
      374,   397,   412,   462,   525,
      561,   640,   724,   837,   924};

//----------------------------------------------------------------------------------------------------------------------
// Pegel-Tabelle der 25 Pegel-Werte von 0 cm bis 120 cm in 5cm-Intervallen:
//----------------------------------------------------------------------------------------------------------------------
__flash S16 p[25] =
  {     0,     5,    10,    15,    20,
       25,    30,    35,    40,    45,
       50,    55,    60,    65,    70,
       75,    80,    85,    90,    95,
      100,   105,   110,   115,   120};

//----------------------------------------------------------------------------------------------------------------------
// Volumen-Tabelle der 25 Volumen-Werte fuer die gemessenen Fuellstaende:
//----------------------------------------------------------------------------------------------------------------------
__flash float v[25]=
  { 0.000, 0.134, 0.272, 0.431, 0.585,
    0.748, 0.908, 1.069, 1.232, 1.397,
    1.560, 1.723, 1.887, 2.054, 2.216,
    2.375, 2.540, 2.699, 2.843, 2.987,
    3.127, 3.259, 3.375, 3.481, 3.581};

//----------------------------------------------------------------------------------------------------------------------

__flash fpU08 LCD_STR1[4] =                       // Display-Text
{
  ("Pegel:            cm"),
  ("Fuellstand:       m3"),
  ("Verbrauch:        m3"),
  ("Ueberlauf:        m3")                       // Anzeige fuer Wirkbetrieb
//  ("ADC-Wert:         --")                       // Anzeige fuer Testbetrieb
};
__flash U08 _SIGN4[8] =
{
  0x0E,0x01,0x06,0x01,0x0E,0x00,0x00,0x00        // Hoch-3-Zeichen
};

//----------------------------------------------------------------------------------------------------------------------
//--Hauptprogramm main()------------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
void main(void)
{

  // Setzen des System-Taktes (CPU-Clock), ADC einschalten (PRADC = 0), Ausschalten nicht benutzter Hardware-Module
  app_init();                                    // Initialisierung der Anwendung (Modul application)

  // Software-Module bereitstellen
  lcd_init();                                    // Initialisierung des LCD-Interface (Modul lcd_2wire)
  adc_init();                                    // Initialisierung des Analog-Digital-Umsetzers
  rwna_init();                                   // Initialisierung der Regenwassernutzungsanlage

//----------------------------------------------------------------------------------------------------------------------
// Setzen des Daten-Richtungs-Registers DDRD 
  DDRD   |= 0b00000011;                          // Bit0 = 1 fuer Relais MAGNET und Bit1 = 1 fuer Relais MOTOR)
  PORTD  |= 0b00000010;                          // PD0 = LOW;  Relais fuer Magnetventil auf AUS
                                                 // PD1 = HIGH; Relais fuer Pumpenaggregat auf EIN
//----------------------------------------------------------------------------------------------------------------------

// Vorbesetzung der Zeilen im Display mit Strings
// Setzen des Sonder-Zeichens Hoch-3 im LCD-Controller
  lcd_download_char_from_rom(4,(fpU08)&_SIGN4);  // auf Adresse 4

  for (i = 0; i < 4; i++)                        // Schreibe Display-Text
  {
    lcd_goto_xy(0,i);                            // Setze Cursor an LCD-Anfang
    lcd_write_flash_str((fpU08)LCD_STR1[i]);     // Begleittext der Zeile
    if (i>0)
    {
      lcd_goto_xy(19,i);                         // An das Ende der Zeilen (ausgenommen Zeile 0)
      lcd_write_char(4);                         // Hoch-3-Zeichen (4 ist die Zeichen-Adresse im CGRAM)
    }
  }

//----------------------------------------------------------------------------------------------------------------------
//--Haupt-Endlosschleife Loop 1-----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------------------------
  while (1)                                      // Loop 1
  {
    nl_messen();                                 // Neuer ADC-Mittelwert vom IRDMS ==> n_lesen

    w_p_v_tabellen();                            // Neuer Pegel- und Volumen-Wert dem ADC-Mittelwert entsprechend
                                                 // aus den Tabellen durch Interpolation entnehmen
                                                 // ==> n_pegel und n_volumen

    ruhe();                                      // Pegelberuhigung erzwingen

    if (vol)                                     // Wenn Schalter VOL gesetzt ist, dann
    {                                            // Verbrauchs-Volumen bzw. Ueberlaufs-Volumen updaten
      volumina();                                // Bei positivem Differenz-Volumen wird die
                                                 // Anzeige Verbrauch ODER Ueberlauf erhoeht.
      a_pegel   = n_pegel;
      a_volumen = n_volumen;
    }

    lcd_display();                               // Alle Werte zur Anzeige bringen
    np_pegel();                                  // Wenn Pegel-Wert >= 120 cm, dann Magnetventil EIN ==> p_120 = 1
                                                 // Wenn Pegel-Wert <= 115 cm, dann Magnetventil AUS ==> p_120 = 0
                                                 // Wenn Pegel-Wert >   15 cm, dann Pumpenaggregat EIN
                                                 // Wenn Pegel-Wert <=  10 cm, dann Pumpenaggregat AUS
  }                                              // Ende der Endlosschleife Loop 1
}                                                // Programmende
//----------------------------------------------------------------------------------------------------------------------
//--Funktion adc_init()                                                 Initialisierung des Analog-Digital-Umsetzers----
//----------------------------------------------------------------------------------------------------------------------

void adc_init(void)                              // Initialisierung des Analog-Digital-Umsetzers
// Adresse 0x78 fuer ADCL     ADC-Datenregister-Low-Byte zur Uebernahme des Wandler-Ergebnisses (untere 8 Bits)
// Adresse 0x79 fuer ADCH     ADC-Datenregister-High-Byte, zur Uebernahme der restlichen 2 hoechsten Bits
// Adresse 0x7A fuer ADCSRA   ADC Control and Status Register A mit Bit7 ADEN, Bit6 ADSC, Bit5 ADATE
// Adresse 0x7B fuer ADCSRB   ADC Control and Status Register B mit ADTS0...ADTS2 Auto Trigger Source Selection
// Adresse 0x7C fuer ADMUX    ADC Multiplexer Selection Register: REFS1=1, REFS0 = 1 => 1,1 Volt Reference intern

{
  ADCSRB = 0;                                    // ADCSRB=00000000              Set ADC to Free Running Trigger
                                                 // ADTS2=0, ADTS1=0, ADTS0=0    Free Running Mode
  ADCSRA = 0x80 | 0x04 | 0x02 | 0x01;            // ADCSRA=10000111              Turn on ADC, setze Prescaler
                                                 // Bit7 ADEN  =1                ADC aktiviert
                                                 // Bit6 ADSC  =0                Konvertierung im Free Running Mode
                                                 // Bit5 ADATE =0                Kein Auto Triggering
                                                 // Bit4 ADIF  =0                Interrupt Flag
                                                 // Bit3 ADIE  =0                Interrupt deaktiviert
                                                 // Bit2 bis Bit0                bestimmen den Teiler (Prescaler):
                                                 //                              System-Takt dividiert durch Prescaler
                                                 //                              ergibt den Eingabe-Takt fuer den
                                                 //                              ADC Clock Source
                                                 // Bit2 ADPS2 =1
                                                 // Bit1 ADPS1 =1
                                                 // Bit0 ADPS0 =1                Teiler = 128
}

//----------------------------------------------------------------------------------------------------------------------
//--Funktion adc_read()                                 Lesen eines Wertes vom Analog-Digital-Umsetzers als ADC-Wert----
//----------------------------------------------------------------------------------------------------------------------

void adc_read(void)                              // Funktion: ADC-Mess-Wert-Erfassung und Umsetzung
{
  ADMUX = 0x80 | 0x40 | 0x05;                    // ADMUX=1100xxxx               Analog-Eingangs-Pin, interne Referenz
                                                 // Bit7 REFS1 =1                Reference Selection Bits:
                                                 // Bit6 REFS0 =1                interne 1,1 V Referenz-Spannung
                                                 // Bit5 ADLAR =0                ADC-Ergebnis rechtsbuendig in ADCL
                                                 // Bit4       =0                Reserviertes Bit stets 0 (nicht belegt)
                                                 // Bit3 bis Bit0                legen den Analog-Eingangs-Pin fest;
                                                 // Bit3 MUX3  =0
                                                 // Bit2 MUX2  =1
                                                 // Bit1 MUX1  =0
                                                 // Bit0 MUX0  =1                Eingangs-Pin ist AD5 (0101)

  ADCSRA |= 0x40;                                // ADCSRA=11000111              Einzel-Konvertierung starten:
                                                 // Bit7 ADEN =1                 ADC aktiviert
                                                 // Bit6 ADSC =1                 Single Conversion Mode
                                                 // Bit5 ADATE=0                 Kein Auto Triggering
                                                 // Bit4 ADIF =0                 Interrupt Flag
                                                 // Bit3 ADIE =0                 Interrupt deaktiviert
                                                 // Bit2 bis Bit0                bestimmen den Teiler (Prescaler):
                                                 //                              System-Takt dividiert durch Prescaler
                                                 //                              ergibt den ADC Clock Source, das ist
                                                 //                              der Eingabe-Takt fuer den ADC
                                                 // Bit2 ADPS2=1
                                                 // Bit1 ADPS1=1
                                                 // Bit0 ADPS0=1                 Teiler = 128


  while (ADCSRA & 0x40)                          // Warte bis Bit6=0, dann Konvertierung abgeschlossen:
  {                                              // Bit6 ADSC wird so lange auf logisch 1 belassen, bis
  }                                              // die Konvertierung abgeschlossen ist. Die fertige
                                                 // Konvertierung uebergibt Bit6=0

  adc_einzel = ADCL + ADCH * 256;                // Low-Byte-Wert + 256*High-Byte-Wert (8-Bit-Shift)
}                                                // Uebergabe des ADC-Mess-Wertes als 16-Bit-Ergebnis

//----------------------------------------------------------------------------------------------------------------------
//--Funktion rwna_init()                                               Initialisierung der Regenwassernutzungsanlage----
//----------------------------------------------------------------------------------------------------------------------
void rwna_init(void)
{
  nl_messen();                                   // Ersten ADC-Mittelwert vom IRDMS ==> n_lesen
  w_p_v_tabellen();                              // Ersten Pegel- und Volumen-Wert dem ADC-Mittelwert entsprechend
                                                 // aus den Tabellen durch Interpolation entnehmen
  a_pegel   = n_pegel;                           // Vorbesetzen des Anfangspegels nach RESET
  a_volumen = n_volumen;                         // Vorbesetzen des Anfangsvolumens nach RESET
}

//----------------------------------------------------------------------------------------------------------------------
//--Funktion nl_messen()                             Ermitteln eines neuen Mittelwertes aus 256 aktuellen Messwerten----
//----------------------------------------------------------------------------------------------------------------------
void nl_messen(void)
{
  n_lesen = 0;
  for (i = 0; i < 8; i++)                        // Loop 2
  {
    adc_mittel = 0;                              // Aktueller Wert der Messreihe
    for (j = 0; j < 32; j++)                     // Loop 3
    {
      adc_read();                                // ADC-Messwert vom IRDMS holen
      adc_mittel += adc_einzel;                  // aufaddieren von 32 Messwerten
      delay_ms(10);                              // Delay - 10 ms bis zum Lesen des naechsten Messwertes
    }
    adc_mittel = adc_mittel >> 5;                // Mittelwert aus der Teil-Summe von 32 Messwerten bilden
    n_lesen += adc_mittel;
  }
  n_lesen = n_lesen >> 3;                        // Mittelwert aus der Gesamt-Summe von 256 Messwerten bilden
}

//----------------------------------------------------------------------------------------------------------------------
//--Funktion w_p_v_tabellen                   Pegel- und Volumenwerte durch Interpolation aus den Tabellen ermitteln----
//----------------------------------------------------------------------------------------------------------------------
void w_p_v_tabellen(void)
{
  if (n_lesen < 180)                            // Pegel-Minimum?
  {
    n_pegel   = 0;                               // Pegel und Volumen auf Minimum setzen
    n_volumen = 0.110;
    return;
  }
  if (n_lesen >= 924)                            // Pegel-Maximum
  {
    n_pegel   = 120;                             // Pegel und Volumen auf Maximum setzen
    n_volumen = 3.581;
    return;
  }
  for (i = 0; i < 24; i++)                       // Loop 4
  {
    w1_tab = w[i];                               // Unterer Interpolationswert aus ADC-Werte-Tabelle
    w2_tab = w[i+1];                             // Oberer Interpolationswert aus ADC-Werte-Tabelle
    p1_tab = p[i];                               // Unterer Interpolationswert aus Pegel-Tabelle
    p2_tab = p[i+1];                             // Oberer Interpolationswert aus Pegel-Tabelle
    v1_tab = v[i];                               // Unterer Interpolationswert aus Volumen-Tabelle
    v2_tab = v[i+1];                             // Oberer Interpolationswert aus Volumen-Tabelle

    if (n_lesen == w1_tab)                       // Gemessener Pegel-Wert ist identisch mit unterem Interpolationswert
    {
      n_pegel   = p1_tab;
      n_volumen = v1_tab;
      return;
    }
    
    if (n_lesen == w2_tab)                       // Gemessener Pegel-Wert ist identisch mit oberem Interpolationswert
    {
      n_pegel   = p2_tab;
      n_volumen = v2_tab;
      return;
    }

    if ((n_lesen > w1_tab) && (n_lesen < w2_tab)) // liegt der gemittelte ADC-Lese-Wert zwischen den Tabellenwerten?
    {                                             // wenn JA, dann Interpolation
      prop_fak  = (float)(n_lesen - w1_tab) / (float)(w2_tab - w1_tab);
      n_pegel   = p1_tab + (S16)(prop_fak*(float)(p2_tab - p1_tab));
      n_volumen = v1_tab + (prop_fak*(v2_tab - v1_tab));
      return;
    }
  }                                              // Loop 4 - Weitere Tabellen-Werte pruefen bis Tabellen-Ende
}
//----------------------------------------------------------------------------------------------------------------------
//--Funktion ruhe()                                     "Erzwingen" der Beruhigung und Plausibilitaet des Messwertes----
//----------------------------------------------------------------------------------------------------------------------
// Der Pegel-Wert schwankt sehr stark, besonders im unteren Pegel-Bereich, wegen der flachen Sensor-Charakteristik
// (die ADC-Werte liegen dicht beieinander), so dass die Wasserentnahme erst bei entsprechender Pegel-Differenz
// gewertet wird. Andernfalls wuerde der Verbrauch wegen der Schwankungen scheinbar ansteigen, ohne dass eine
// Wasserentnahme stattgefunden hat.

  void ruhe(void)
// Wenn n_pegel groesser/gleich a_pegel, dann ist Wasser ergaenzt worden:
{
  if (n_pegel >= a_pegel)                        // Vergleichen des neuen Wertes mit dem vorherigen Wert
  {
    a_pegel   = n_pegel;
    a_volumen = n_volumen;
    return;
  }                                              // Andernfalls ist ggf. Wasser entnommen worden und es muss die
                                                 // Beruhigung abgewartet werden:
// Bei Pegel-Werten groesser 80 cm treten nur geringe Schwankungen auf:
  if ((n_pegel >= 80) && ((a_pegel - n_pegel) >= 5))
                                                 // Wenn der a_pegel um 5 cm oder mehr groesser als der n_pegel ist,
  {                                              // dann ist Wasser entnommen worden.
    vol = 1;                                     // Logischer Schalter fuer Entnahme auf TRUE
    return;
  }

// Bei einem Pegel unter 80 cm wird erst ab einer Pegel-Differenz von 10 cm eine Entnahme registriert
  if ((n_pegel < 80) && ((a_pegel - n_pegel) >= 10))  // Wenn JA, dann wird eine Wasserentnahme unterstellt
  {
    vol = 1;                                     // Logischer Schalter fuer Entnahme auf TRUE
    return;
  }
// Ob Gartenwasser verbraucht wurde oder ob Ueberlauf in den Gartenteich erfolgte, richtet sich nach dem Pegel=120 cm
}
//----------------------------------------------------------------------------------------------------------------------
//--Funktion volumina()                        Wasserverbrauch auf Gartenwasser oder Teich zuordnen und aufsummieren----
//----------------------------------------------------------------------------------------------------------------------
  void volumina(void)
{

  vol = 0;                                       // Schalter fuer Wasserentnahme wieder auf Null setzen

  d_volumen = a_volumen - n_volumen;             // Die Differenz ist die Menge des entnommenen Wassers

  if (p_120)
  {
    t_volumen += d_volumen;                      // Differenz dem Ueberlauf-Volumen zuschlagen
    return;
  }

  g_volumen += d_volumen;                        // bzw. der Gartenbewaesserung zuschlagen
}

//----------------------------------------------------------------------------------------------------------------------
//--Funktion lcd_display()                    Ausgeben der aktuellen Messwerte; Verbrauch und Ueberlauf unveraendert----
//----------------------------------------------------------------------------------------------------------------------
void lcd_display(void)
{
  lcd_goto_xy(12,0);                             // Setze Cursor in der 1. LCD-Zeile auf 12
  lcd_write_S16(n_pegel,5);                      // Pegel-Wert
  lcd_goto_xy(11,1);                             // Setze Cursor in der 2. LCD-Zeile auf 11
  lcd_write_float(n_volumen,2,6);                // Volumen-Wert fuer den Fuellstand
  lcd_goto_xy(11,2);                             // Setze Cursor in der 3. LCD-Zeile auf 11
  lcd_write_float(g_volumen,2,6);                // Volumen-Wert fuer Verbrauch an Gartenwasser
  lcd_goto_xy(11,3);                             // Setze Cursor in der 4. LCD-Zeile auf 11         entfaellt beim Tests
  lcd_write_float(t_volumen,2,6);                // Volumen-Wert fuer Ueberlauf in den Gartenteich  entfaellt beim Tests
//  lcd_write_char_xy(19,3,'-');                   // Zeile fuer Testzwecke (keine Dimension)
//  lcd_write_char_xy(18,3,'-');                   // Zeile fuer Testzwecke (keine Dimension)
//  lcd_goto_xy(12,3);                             // Zeile fuer Testzwecke: Setze Cursor in der 4. LCD-Zeile auf 12
//  lcd_write_S16(n_lesen,5);                      // Zeile fuer Testzwecke: ADC-Lesewert anzeigen und kontrollieren
}

//----------------------------------------------------------------------------------------------------------------------
//--Funktion np_pegel()                                                         Ueberlauf und Leerlaufschutz steuern----
//----------------------------------------------------------------------------------------------------------------------
  void np_pegel(void)
{
  if (n_pegel >= 120)                            // Wenn der Pegel groesser oder gleich 120cm ist,
  {
    p_120  = 1;                                  // dann setze logischen Schalter fuer Pegel 120 cm und
    PORTD |= 0b00000001;                         // schalte Magnetventil auf EIN (Ueberlauf)
    return;
  }

  if (n_pegel <= 115)                            // Magnetventil bleibt auf EIN bis Pegel kleiner oder gleich 115 cm
  {
    p_120  = 0;                                  // Wenn Pegel kleiner oder gleich 115 cm, dann setze logischen
    PORTD &= 0b11111110;                         // Schalter fuer Pegel 120 cm wieder auf 0 und Magnetventil auf AUS

    if (n_pegel >= 15)                           // Pegel groesser/gleich Reserve von 15 cm? Wenn JA, dann
    {
      PORTD |= 0b00000010;                       // Relais fuer Pumpenaggregat auf EIN
      return;                                    // Reserve verhindert staendiges EIN-/AUS-Schalten des Pumpenaggregats
    }                                            // auf Grund von Schwankungen im unteren Pegel-Bereich 
    if (n_pegel <= 10)                           // Wenn Pegel kleiner/gleich 10 cm, dann
    {
      PORTD &= 0b11111101;                       // Relais fuer Pumpenaggregat auf AUS (Leerlaufschutz)
      n_pegel   = 0;                             // Pegel und Volumen auf Null setzen
      return;
    }
  }
}

//----------------------------------------------------------------------------------------------------------------------
